@@ -12,6 +12,12 @@ use serde::Serialize;
 use ulid::Ulid;
 use url::Url;
 
+#[derive(Debug, Clone, PartialEq, Eq, Serialize)]
+pub struct MatrixUser {
+    pub mxid: String,
+    pub display_name: Option<String>,
+}
+
 #[derive(Debug, Clone, PartialEq, Eq, Serialize)]
 pub struct User {
     pub id: Ulid,
@@ -272,6 +278,7 @@ pub struct UserRegistration {
     pub email_authentication_id: Option<Ulid>,
     pub user_registration_token_id: Option<Ulid>,
     pub password: Option<UserRegistrationPassword>,
+    pub upstream_oauth_authorization_session_id: Option<Ulid>,
     pub post_auth_action: Option<serde_json::Value>,
     pub ip_address: Option<IpAddr>,
     pub user_agent: Option<String>,
