@@ -29,6 +29,7 @@ use mas_router::{
     UrlBuilder,
 };
 use mas_templates::{ApiDocContext, Templates};
+use schemars::transform::AddNullable;
 use tower_http::cors::{Any, CorsLayer};
 
 mod call_context;
@@ -171,8 +172,16 @@ where
     aide::generate::infer_responses(false);
 
     aide::generate::in_context(|ctx| {
-        ctx.schema =
-            schemars::r#gen::SchemaGenerator::new(schemars::r#gen::SchemaSettings::openapi3());
+        ctx.schema = schemars::generate::SchemaGenerator::new(
+            schemars::generate::SchemaSettings::openapi3().with(|settings| {
+                // Remove the transform which adds nullable fields, as it's not
+                // valid with OpenAPI 3.1. For some reason, aide/schemars output
+                // an OpenAPI 3.1 schema with this nullable transform.
+                settings
+                    .transforms
+                    .retain(|transform| !transform.is::<AddNullable>());
+            }),
+        );
     });
 
     let mut api = OpenApi::default();
