@@ -12,6 +12,7 @@ use mas_data_model::{BrowserSession, Client, Clock, Session, SessionState, User}
 use mas_storage::{
     Page, Pagination,
     oauth2::{OAuth2SessionFilter, OAuth2SessionRepository},
+    pagination::Node,
 };
 use oauth2_types::scope::{Scope, ScopeToken};
 use rand::RngCore;
@@ -61,6 +62,12 @@ struct OAuthSessionLookup {
     human_name: Option<String>,
 }
 
+impl Node<Ulid> for OAuthSessionLookup {
+    fn cursor(&self) -> Ulid {
+        self.oauth2_session_id.into()
+    }
+}
+
 impl TryFrom<OAuthSessionLookup> for Session {
     type Error = DatabaseInconsistencyError;
 
