@@ -21,19 +21,22 @@ use chrono::{DateTime, Duration, Utc};
 use http::{Method, Uri, Version};
 use mas_data_model::{
     AuthorizationGrant, BrowserSession, Client, CompatSsoLogin, CompatSsoLoginState,
-    DeviceCodeGrant, UpstreamOAuthLink, UpstreamOAuthProvider, UpstreamOAuthProviderClaimsImports,
-    UpstreamOAuthProviderDiscoveryMode, UpstreamOAuthProviderOnBackchannelLogout,
-    UpstreamOAuthProviderPkceMode, UpstreamOAuthProviderTokenAuthMethod, User,
-    UserEmailAuthentication, UserEmailAuthenticationCode, UserRecoverySession, UserRegistration,
+    DeviceCodeGrant, MatrixUser, UpstreamOAuthLink, UpstreamOAuthProvider,
+    UpstreamOAuthProviderClaimsImports, UpstreamOAuthProviderDiscoveryMode,
+    UpstreamOAuthProviderOnBackchannelLogout, UpstreamOAuthProviderPkceMode,
+    UpstreamOAuthProviderTokenAuthMethod, User, UserEmailAuthentication,
+    UserEmailAuthenticationCode, UserRecoverySession, UserRegistration,
 };
 use mas_i18n::DataLocale;
 use mas_iana::jose::JsonWebSignatureAlg;
+use mas_policy::{Violation, ViolationCode};
 use mas_router::{Account, GraphQL, PostAuthAction, UrlBuilder};
 use oauth2_types::scope::{OPENID, Scope};
 use rand::{
-    Rng,
+    Rng, SeedableRng,
     distributions::{Alphanumeric, DistString},
 };
+use rand_chacha::ChaCha8Rng;
 use serde::{Deserialize, Serialize, ser::SerializeStruct};
 use ulid::Ulid;
 use url::Url;
@@ -106,9 +109,9 @@ pub trait TemplateContext: Serialize {
     ///
     /// This is then used to check for template validity in unit tests and in
     /// the CLI (`cargo run -- templates check`)
-    fn sample(
+    fn sample<R: Rng>(
         now: chrono::DateTime<Utc>,
-        rng: &mut impl Rng,
+        rng: &mut R,
         locales: &[DataLocale],
     ) -> BTreeMap<SampleIdentifier, Self>
     where
@@ -144,9 +147,9 @@ pub(crate) fn sample_list<T: TemplateContext>(samples: Vec<T>) -> BTreeMap<Sampl
 }
 
 impl TemplateContext for () {
-    fn sample(
+    fn sample<R: Rng>(
         _now: chrono::DateTime<Utc>,
-        _rng: &mut impl Rng,
+        _rng: &mut R,
         _locales: &[DataLocale],
     ) -> BTreeMap<SampleIdentifier, Self>
     where
@@ -181,18 +184,20 @@ impl<T> std::ops::Deref for WithLanguage<T> {
 }
 
 impl<T: TemplateContext> TemplateContext for WithLanguage<T> {
-    fn sample(
+    fn sample<R: Rng>(
         now: chrono::DateTime<Utc>,
-        rng: &mut impl Rng,
+        rng: &mut R,
         locales: &[DataLocale],
     ) -> BTreeMap<SampleIdentifier, Self>
     where
         Self: Sized,
     {
+        // Create a forked RNG so we make samples deterministic between locales
+        let rng = ChaCha8Rng::from_rng(rng).unwrap();
         locales
             .iter()
             .flat_map(|locale| {
-                T::sample(now, rng, locales)
+                T::sample(now, &mut rng.clone(), locales)
                     .into_iter()
                     .map(|(sample_id, sample)| {
                         (
@@ -218,9 +223,9 @@ pub struct WithCsrf<T> {
 }
 
 impl<T: TemplateContext> TemplateContext for WithCsrf<T> {
-    fn sample(
+    fn sample<R: Rng>(
         now: chrono::DateTime<Utc>,
-        rng: &mut impl Rng,
+        rng: &mut R,
         locales: &[DataLocale],
     ) -> BTreeMap<SampleIdentifier, Self>
     where
@@ -251,9 +256,9 @@ pub struct WithSession<T> {
 }
 
 impl<T: TemplateContext> TemplateContext for WithSession<T> {
-    fn sample(
+    fn sample<R: Rng>(
         now: chrono::DateTime<Utc>,
-        rng: &mut impl Rng,
+        rng: &mut R,
         locales: &[DataLocale],
     ) -> BTreeMap<SampleIdentifier, Self>
     where
@@ -289,9 +294,9 @@ pub struct WithOptionalSession<T> {
 }
 
 impl<T: TemplateContext> TemplateContext for WithOptionalSession<T> {
-    fn sample(
+    fn sample<R: Rng>(
         now: chrono::DateTime<Utc>,
-        rng: &mut impl Rng,
+        rng: &mut R,
         locales: &[DataLocale],
     ) -> BTreeMap<SampleIdentifier, Self>
     where
@@ -340,9 +345,9 @@ impl Serialize for EmptyContext {
 }
 
 impl TemplateContext for EmptyContext {
-    fn sample(
+    fn sample<R: Rng>(
         _now: chrono::DateTime<Utc>,
-        _rng: &mut impl Rng,
+        _rng: &mut R,
         _locales: &[DataLocale],
     ) -> BTreeMap<SampleIdentifier, Self>
     where
@@ -368,9 +373,9 @@ impl IndexContext {
 }
 
 impl TemplateContext for IndexContext {
-    fn sample(
+    fn sample<R: Rng>(
         _now: chrono::DateTime<Utc>,
-        _rng: &mut impl Rng,
+        _rng: &mut R,
         _locales: &[DataLocale],
     ) -> BTreeMap<SampleIdentifier, Self>
     where
@@ -414,9 +419,9 @@ impl AppContext {
 }
 
 impl TemplateContext for AppContext {
-    fn sample(
+    fn sample<R: Rng>(
         _now: chrono::DateTime<Utc>,
-        _rng: &mut impl Rng,
+        _rng: &mut R,
         _locales: &[DataLocale],
     ) -> BTreeMap<SampleIdentifier, Self>
     where
@@ -447,9 +452,9 @@ impl ApiDocContext {
 }
 
 impl TemplateContext for ApiDocContext {
-    fn sample(
+    fn sample<R: Rng>(
         _now: chrono::DateTime<Utc>,
-        _rng: &mut impl Rng,
+        _rng: &mut R,
         _locales: &[DataLocale],
     ) -> BTreeMap<SampleIdentifier, Self>
     where
@@ -539,9 +544,9 @@ pub struct LoginContext {
 }
 
 impl TemplateContext for LoginContext {
-    fn sample(
+    fn sample<R: Rng>(
         _now: chrono::DateTime<Utc>,
-        _rng: &mut impl Rng,
+        _rng: &mut R,
         _locales: &[DataLocale],
     ) -> BTreeMap<SampleIdentifier, Self>
     where
@@ -647,9 +652,9 @@ pub struct RegisterContext {
 }
 
 impl TemplateContext for RegisterContext {
-    fn sample(
+    fn sample<R: Rng>(
         _now: chrono::DateTime<Utc>,
-        _rng: &mut impl Rng,
+        _rng: &mut R,
         _locales: &[DataLocale],
     ) -> BTreeMap<SampleIdentifier, Self>
     where
@@ -690,9 +695,9 @@ pub struct PasswordRegisterContext {
 }
 
 impl TemplateContext for PasswordRegisterContext {
-    fn sample(
+    fn sample<R: Rng>(
         _now: chrono::DateTime<Utc>,
-        _rng: &mut impl Rng,
+        _rng: &mut R,
         _locales: &[DataLocale],
     ) -> BTreeMap<SampleIdentifier, Self>
     where
@@ -729,12 +734,13 @@ pub struct ConsentContext {
     grant: AuthorizationGrant,
     client: Client,
     action: PostAuthAction,
+    matrix_user: MatrixUser,
 }
 
 impl TemplateContext for ConsentContext {
-    fn sample(
+    fn sample<R: Rng>(
         now: chrono::DateTime<Utc>,
-        rng: &mut impl Rng,
+        rng: &mut R,
         _locales: &[DataLocale],
     ) -> BTreeMap<SampleIdentifier, Self>
     where
@@ -752,6 +758,10 @@ impl TemplateContext for ConsentContext {
                         grant,
                         client,
                         action,
+                        matrix_user: MatrixUser {
+                            mxid: "@alice:example.com".to_owned(),
+                            display_name: Some("Alice".to_owned()),
+                        },
                     }
                 })
                 .collect(),
@@ -762,12 +772,13 @@ impl TemplateContext for ConsentContext {
 impl ConsentContext {
     /// Constructs a context for the client consent page
     #[must_use]
-    pub fn new(grant: AuthorizationGrant, client: Client) -> Self {
+    pub fn new(grant: AuthorizationGrant, client: Client, matrix_user: MatrixUser) -> Self {
         let action = PostAuthAction::continue_grant(grant.id);
         Self {
             grant,
             client,
             action,
+            matrix_user,
         }
     }
 }
@@ -790,9 +801,9 @@ pub struct PolicyViolationContext {
 }
 
 impl TemplateContext for PolicyViolationContext {
-    fn sample(
+    fn sample<R: Rng>(
         now: chrono::DateTime<Utc>,
-        rng: &mut impl Rng,
+        rng: &mut R,
         _locales: &[DataLocale],
     ) -> BTreeMap<SampleIdentifier, Self>
     where
@@ -857,40 +868,89 @@ impl PolicyViolationContext {
     }
 }
 
+/// Context used by the `compat_login_policy_violation.html` template
+#[derive(Serialize)]
+pub struct CompatLoginPolicyViolationContext {
+    violations: Vec<Violation>,
+}
+
+impl TemplateContext for CompatLoginPolicyViolationContext {
+    fn sample<R: Rng>(
+        _now: chrono::DateTime<Utc>,
+        _rng: &mut R,
+        _locales: &[DataLocale],
+    ) -> BTreeMap<SampleIdentifier, Self>
+    where
+        Self: Sized,
+    {
+        sample_list(vec![
+            CompatLoginPolicyViolationContext { violations: vec![] },
+            CompatLoginPolicyViolationContext {
+                violations: vec![Violation {
+                    msg: "user has too many active sessions".to_owned(),
+                    redirect_uri: None,
+                    field: None,
+                    code: Some(ViolationCode::TooManySessions),
+                }],
+            },
+        ])
+    }
+}
+
+impl CompatLoginPolicyViolationContext {
+    /// Constructs a context for the compatibility login policy violation page
+    /// given the list of violations
+    #[must_use]
+    pub const fn for_violations(violations: Vec<Violation>) -> Self {
+        Self { violations }
+    }
+}
+
 /// Context used by the `sso.html` template
 #[derive(Serialize)]
 pub struct CompatSsoContext {
     login: CompatSsoLogin,
     action: PostAuthAction,
+    matrix_user: MatrixUser,
 }
 
 impl TemplateContext for CompatSsoContext {
-    fn sample(
+    fn sample<R: Rng>(
         now: chrono::DateTime<Utc>,
-        rng: &mut impl Rng,
+        rng: &mut R,
         _locales: &[DataLocale],
     ) -> BTreeMap<SampleIdentifier, Self>
     where
         Self: Sized,
     {
         let id = Ulid::from_datetime_with_source(now.into(), rng);
-        sample_list(vec![CompatSsoContext::new(CompatSsoLogin {
-            id,
-            redirect_uri: Url::parse("https://app.element.io/").unwrap(),
-            login_token: "abcdefghijklmnopqrstuvwxyz012345".into(),
-            created_at: now,
-            state: CompatSsoLoginState::Pending,
-        })])
+        sample_list(vec![CompatSsoContext::new(
+            CompatSsoLogin {
+                id,
+                redirect_uri: Url::parse("https://app.element.io/").unwrap(),
+                login_token: "abcdefghijklmnopqrstuvwxyz012345".into(),
+                created_at: now,
+                state: CompatSsoLoginState::Pending,
+            },
+            MatrixUser {
+                mxid: "@alice:example.com".to_owned(),
+                display_name: Some("Alice".to_owned()),
+            },
+        )])
     }
 }
 
 impl CompatSsoContext {
     /// Constructs a context for the legacy SSO login page
     #[must_use]
-    pub fn new(login: CompatSsoLogin) -> Self
+    pub fn new(login: CompatSsoLogin, matrix_user: MatrixUser) -> Self
 where {
         let action = PostAuthAction::continue_compat_sso_login(login.id);
-        Self { login, action }
+        Self {
+            login,
+            action,
+            matrix_user,
+        }
     }
 }
 
@@ -927,9 +987,9 @@ impl EmailRecoveryContext {
 }
 
 impl TemplateContext for EmailRecoveryContext {
-    fn sample(
+    fn sample<R: Rng>(
         now: chrono::DateTime<Utc>,
-        rng: &mut impl Rng,
+        rng: &mut R,
         _locales: &[DataLocale],
     ) -> BTreeMap<SampleIdentifier, Self>
     where
@@ -992,9 +1052,9 @@ impl EmailVerificationContext {
 }
 
 impl TemplateContext for EmailVerificationContext {
-    fn sample(
+    fn sample<R: Rng>(
         now: chrono::DateTime<Utc>,
-        rng: &mut impl Rng,
+        rng: &mut R,
         _locales: &[DataLocale],
     ) -> BTreeMap<SampleIdentifier, Self>
     where
@@ -1067,9 +1127,9 @@ impl RegisterStepsVerifyEmailContext {
 }
 
 impl TemplateContext for RegisterStepsVerifyEmailContext {
-    fn sample(
+    fn sample<R: Rng>(
         now: chrono::DateTime<Utc>,
-        rng: &mut impl Rng,
+        rng: &mut R,
         _locales: &[DataLocale],
     ) -> BTreeMap<SampleIdentifier, Self>
     where
@@ -1107,9 +1167,9 @@ impl RegisterStepsEmailInUseContext {
 }
 
 impl TemplateContext for RegisterStepsEmailInUseContext {
-    fn sample(
+    fn sample<R: Rng>(
         _now: chrono::DateTime<Utc>,
-        _rng: &mut impl Rng,
+        _rng: &mut R,
         _locales: &[DataLocale],
     ) -> BTreeMap<SampleIdentifier, Self>
     where
@@ -1162,9 +1222,9 @@ impl RegisterStepsDisplayNameContext {
 }
 
 impl TemplateContext for RegisterStepsDisplayNameContext {
-    fn sample(
+    fn sample<R: Rng>(
         _now: chrono::DateTime<chrono::Utc>,
-        _rng: &mut impl Rng,
+        _rng: &mut R,
         _locales: &[DataLocale],
     ) -> BTreeMap<SampleIdentifier, Self>
     where
@@ -1217,9 +1277,9 @@ impl RegisterStepsRegistrationTokenContext {
 }
 
 impl TemplateContext for RegisterStepsRegistrationTokenContext {
-    fn sample(
+    fn sample<R: Rng>(
         _now: chrono::DateTime<chrono::Utc>,
-        _rng: &mut impl Rng,
+        _rng: &mut R,
         _locales: &[DataLocale],
     ) -> BTreeMap<SampleIdentifier, Self>
     where
@@ -1268,9 +1328,9 @@ impl RecoveryStartContext {
 }
 
 impl TemplateContext for RecoveryStartContext {
-    fn sample(
+    fn sample<R: Rng>(
         _now: chrono::DateTime<Utc>,
-        _rng: &mut impl Rng,
+        _rng: &mut R,
         _locales: &[DataLocale],
     ) -> BTreeMap<SampleIdentifier, Self>
     where
@@ -1310,9 +1370,9 @@ impl RecoveryProgressContext {
 }
 
 impl TemplateContext for RecoveryProgressContext {
-    fn sample(
+    fn sample<R: Rng>(
         now: chrono::DateTime<Utc>,
-        rng: &mut impl Rng,
+        rng: &mut R,
         _locales: &[DataLocale],
     ) -> BTreeMap<SampleIdentifier, Self>
     where
@@ -1356,9 +1416,9 @@ impl RecoveryExpiredContext {
 }
 
 impl TemplateContext for RecoveryExpiredContext {
-    fn sample(
+    fn sample<R: Rng>(
         now: chrono::DateTime<Utc>,
-        rng: &mut impl Rng,
+        rng: &mut R,
         _locales: &[DataLocale],
     ) -> BTreeMap<SampleIdentifier, Self>
     where
@@ -1420,9 +1480,9 @@ impl RecoveryFinishContext {
 }
 
 impl TemplateContext for RecoveryFinishContext {
-    fn sample(
+    fn sample<R: Rng>(
         now: chrono::DateTime<Utc>,
-        rng: &mut impl Rng,
+        rng: &mut R,
         _locales: &[DataLocale],
     ) -> BTreeMap<SampleIdentifier, Self>
     where
@@ -1453,7 +1513,7 @@ impl TemplateContext for RecoveryFinishContext {
     }
 }
 
-/// Context used by the `pages/upstream_oauth2/{link_mismatch,login_link}.html`
+/// Context used by the `pages/upstream_oauth2/link_mismatch.html`
 /// templates
 #[derive(Serialize)]
 pub struct UpstreamExistingLinkContext {
@@ -1469,9 +1529,9 @@ impl UpstreamExistingLinkContext {
 }
 
 impl TemplateContext for UpstreamExistingLinkContext {
-    fn sample(
+    fn sample<R: Rng>(
         now: chrono::DateTime<Utc>,
-        rng: &mut impl Rng,
+        rng: &mut R,
         _locales: &[DataLocale],
     ) -> BTreeMap<SampleIdentifier, Self>
     where
@@ -1507,9 +1567,9 @@ impl UpstreamSuggestLink {
 }
 
 impl TemplateContext for UpstreamSuggestLink {
-    fn sample(
+    fn sample<R: Rng>(
         now: chrono::DateTime<Utc>,
-        rng: &mut impl Rng,
+        rng: &mut R,
         _locales: &[DataLocale],
     ) -> BTreeMap<SampleIdentifier, Self>
     where
@@ -1636,9 +1696,9 @@ impl UpstreamRegister {
 }
 
 impl TemplateContext for UpstreamRegister {
-    fn sample(
+    fn sample<R: Rng>(
         now: chrono::DateTime<Utc>,
-        _rng: &mut impl Rng,
+        _rng: &mut R,
         _locales: &[DataLocale],
     ) -> BTreeMap<SampleIdentifier, Self>
     where
@@ -1722,9 +1782,9 @@ impl DeviceLinkContext {
 }
 
 impl TemplateContext for DeviceLinkContext {
-    fn sample(
+    fn sample<R: Rng>(
         _now: chrono::DateTime<Utc>,
-        _rng: &mut impl Rng,
+        _rng: &mut R,
         _locales: &[DataLocale],
     ) -> BTreeMap<SampleIdentifier, Self>
     where
@@ -1745,20 +1805,25 @@ impl TemplateContext for DeviceLinkContext {
 pub struct DeviceConsentContext {
     grant: DeviceCodeGrant,
     client: Client,
+    matrix_user: MatrixUser,
 }
 
 impl DeviceConsentContext {
     /// Constructs a new context with an existing linked user
     #[must_use]
-    pub fn new(grant: DeviceCodeGrant, client: Client) -> Self {
-        Self { grant, client }
+    pub fn new(grant: DeviceCodeGrant, client: Client, matrix_user: MatrixUser) -> Self {
+        Self {
+            grant,
+            client,
+            matrix_user,
+        }
     }
 }
 
 impl TemplateContext for DeviceConsentContext {
-    fn sample(
+    fn sample<R: Rng>(
         now: chrono::DateTime<Utc>,
-        rng: &mut impl Rng,
+        rng: &mut R,
         _locales: &[DataLocale],
     ) -> BTreeMap<SampleIdentifier, Self>
     where
@@ -1779,7 +1844,14 @@ impl TemplateContext for DeviceConsentContext {
                     ip_address: Some(IpAddr::V4(Ipv4Addr::LOCALHOST)),
                     user_agent: Some("Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/93.0.0.0 Safari/537.36".to_owned()),
                 };
-                Self { grant, client }
+                Self {
+                    grant,
+                    client,
+                    matrix_user: MatrixUser {
+                        mxid: "@alice:example.com".to_owned(),
+                        display_name: Some("Alice".to_owned()),
+                    }
+                }
             })
             .collect())
     }
@@ -1801,9 +1873,9 @@ impl AccountInactiveContext {
 }
 
 impl TemplateContext for AccountInactiveContext {
-    fn sample(
+    fn sample<R: Rng>(
         now: chrono::DateTime<Utc>,
-        rng: &mut impl Rng,
+        rng: &mut R,
         _locales: &[DataLocale],
     ) -> BTreeMap<SampleIdentifier, Self>
     where
@@ -1837,9 +1909,9 @@ impl DeviceNameContext {
 }
 
 impl TemplateContext for DeviceNameContext {
-    fn sample(
+    fn sample<R: Rng>(
         now: chrono::DateTime<Utc>,
-        rng: &mut impl Rng,
+        rng: &mut R,
         _locales: &[DataLocale],
     ) -> BTreeMap<SampleIdentifier, Self>
     where
@@ -1863,9 +1935,9 @@ pub struct FormPostContext<T> {
 }
 
 impl<T: TemplateContext> TemplateContext for FormPostContext<T> {
-    fn sample(
+    fn sample<R: Rng>(
         now: chrono::DateTime<Utc>,
-        rng: &mut impl Rng,
+        rng: &mut R,
         locales: &[DataLocale],
     ) -> BTreeMap<SampleIdentifier, Self>
     where
@@ -1945,9 +2017,9 @@ impl std::fmt::Display for ErrorContext {
 }
 
 impl TemplateContext for ErrorContext {
-    fn sample(
+    fn sample<R: Rng>(
         _now: chrono::DateTime<Utc>,
-        _rng: &mut impl Rng,
+        _rng: &mut R,
         _locales: &[DataLocale],
     ) -> BTreeMap<SampleIdentifier, Self>
     where
@@ -2039,9 +2111,9 @@ impl NotFoundContext {
 }
 
 impl TemplateContext for NotFoundContext {
-    fn sample(
+    fn sample<R: Rng>(
         _now: DateTime<Utc>,
-        _rng: &mut impl Rng,
+        _rng: &mut R,
         _locales: &[DataLocale],
     ) -> BTreeMap<SampleIdentifier, Self>
     where
